import fs from 'fs';
import ora from 'ora';
import { copyrightSnippet } from './copyright.js';
import { deleteFolderContent, getBoundedContextModules } from '../functions/index.js';
import { clearFolder } from '../helpers/fileOperations.js';
import { purpleColor, stopSpinner, greenColor, TAB, redColor } from '../utils/oraUtils.js';
import { inquirerFuzzy as inquirerPath, inquirerSimpleConfirm, printError, } from '../utils/inquirer.js';
import { transpileCode } from '../functions/transpile.js';
import { writeTranspiledCode } from '../functions/generateTargetFiles.js';
import path from 'path';
const SETUP_FILE_EXTENSION = 'setup.bl';
const questions = [
    {
        name: 'sourceDirPath',
        message: '📦 Where are your source files located?',
        default: '.',
    },
    {
        name: 'targetDirPath',
        message: '🎯 Where would you like your generated files to be exported?',
        default: './output',
    },
];
const transpile = async (source) => {
    console.log();
    console.log(copyrightSnippet);
    console.log();
    const answers = [];
    for (const q of questions) {
        answers.push(await inquirerPath(q, source));
    }
    const [sourceDirPath, targetDirPath] = answers;
    const libTargetDirPath = path.join(targetDirPath, 'lib');
    if (!fs.existsSync(libTargetDirPath)) {
        fs.mkdirSync(libTargetDirPath, { recursive: true });
    }
    else {
        const overwriteAccepted = source.y ||
            (await inquirerSimpleConfirm('Overwrite', '⚠️  Target directory already exists. Overwrite?', 'n'));
        if (!overwriteAccepted) {
            return;
        }
        await deleteFolderContent(libTargetDirPath);
    }
    let throbber;
    try {
        const boundedContextModules = getBoundedContextModules(sourceDirPath);
        const dirEntries = fs.readdirSync(sourceDirPath, { withFileTypes: true });
        const filesNames = dirEntries
            .filter((entry) => entry.isFile())
            .map((fileName) => fileName.name);
        const atLeastOneSetupFileExists = filesNames.some((file) => file.toLowerCase().endsWith(SETUP_FILE_EXTENSION));
        const atLeastOneModuleExists = Object.values(boundedContextModules).some((modules) => modules.length > 0);
        if (!atLeastOneSetupFileExists) {
            printError('No setup file found. Please create a setup file in the root of your project.');
            process.exit(1);
        }
        if (!atLeastOneModuleExists) {
            printError('No modules found. Please create a module inside one of your bounded contexts.');
            process.exit(1);
        }
        clearFolder(targetDirPath);
        throbber = ora(purpleColor('🔨 Transpiling... ')).start();
        const transpiledCode = await transpileCode(boundedContextModules, sourceDirPath);
        stopSpinner(throbber, greenColor('Transpiled'), '🔨');
        throbber = ora(purpleColor('🕒 Writing system files to disk...')).start();
        writeTranspiledCode(transpiledCode, targetDirPath);
        stopSpinner(throbber, greenColor('System files written'), '⏰');
        console.log(greenColor(TAB + '🦎 Project generated successfully!\n'));
    }
    catch (err) {
        console.log(err);
        console.error(redColor(TAB + '❌ ' + err));
        throbber?.stop();
    }
};
export default transpile;
